searchData={"items":[{"type":"module","title":"Chord","doc":"Chord: A powerful library for managing real-time contexts with efficiency and flexibility.\n\nThe Chord module serves as the entry point for the library, providing a high-level API\nto manage contexts, deltas, cleanup, and integrations with various storage backends.\nIt simplifies state synchronization and lifecycle management for real-time applications.","ref":"Chord.html"},{"type":"module","title":"Key Features - Chord","doc":"- **Context Management**: Create, update, synchronize, and delete application contexts.\n- **Delta Tracking**: Efficiently track changes to minimize data transfer.\n- **Flexible Backends**: Support for in-memory (ETS) and distributed (Redis) storage.\n- **Context Export**: Easily export contexts to external storage.\n- **Customizable Cleanup**: Automated or manual cleanup of expired data.\n- **Partial Updates**: Apply updates to specific fields within a context.\n- **External Context Provider**: Restore contexts from external sources when needed.","ref":"Chord.html#module-key-features"},{"type":"module","title":"Configuration - Chord","doc":"Chord is designed to be configurable via the application environment. Below are the\nconfiguration options available:\n\n    config :chord,\n      backend: Chord.Backend.ETS, # Backend for storing contexts and deltas (ETS or Redis)\n      context_auto_delete: true, # Automatically delete expired contexts\n      context_ttl: :timer.hours(6), # Time-to-live for contexts\n      delta_ttl: :timer.hours(3), # Time-to-live for deltas\n      delta_threshold: 100, # Maximum number of deltas to retain per context\n      delta_formatter: Chord.Delta.Formatter.Default, # Formatter for deltas\n      time_provider: MyApp.TimeProvider, # Custom time provider for timestamping\n      export_callback: &MyApp.ContextExporter.export/1, # Callback for exporting contexts\n      context_external_provider: &MyApp.ExternalState.fetch_context/1 # Restore context from external storage","ref":"Chord.html#module-configuration"},{"type":"module","title":"Common Options - Chord","doc":"The following options are shared across several functions in this module and related modules:\n\n  - :context_id - The identifier for the context.\n  - :version - The version number for tracking state changes.\n  - :inserted_at - The timestamp when the context or delta was created.\n  - :limit - The maximum number of contexts or deltas to fetch.\n  - :offset - The number of entries to skip when fetching.\n  - :order - The sort order for results (:asc or :desc).","ref":"Chord.html#module-common-options"},{"type":"module","title":"Example Usage - Chord","doc":"Here's how you can use Chord in your application:\n\n    # Set a context\n    {:ok, %{context: updated_context, delta: delta}} =\n      Chord.set_context(\"call:123\", %{status: \"active\"})\n\n    # Get the current context\n    {:ok, %{context_id: context_id, context: context, version: version, inserted_at: inserted_at}} = Chord.get_context(\"call:123\")\n\n    # Apply a partial update to a context\n    {:ok, %{context: updated_context, delta: delta}} =\n      Chord.update_context(\"call:123\", %{status: \"ended\"})\n\n    # Synchronize context with a client\n    case Chord.sync_context(\"call:123\", client_version) do\n      {:full_context, context} -> :send_full_context\n      {:delta, delta} -> :send_delta\n      {:no_change, version} -> :no_change\n    end\n\n    # Restore a context from external storage\n    {:ok, context_data} = Chord.restore_context(\"call:123\")\n\n    # Export a context to external storage\n    :ok = Chord.export_context(\"call:123\")\n\n    # Trigger cleanup\n    Chord.cleanup(limit: 10)\n\n    # Start the Cleanup.Server for periodic cleanup\n    {:ok, _pid} = Chord.start_cleanup_server(interval: :timer.minutes(30))","ref":"Chord.html#module-example-usage"},{"type":"function","title":"Chord.cleanup/1","doc":"Triggers a one-time cleanup for stale contexts and deltas.","ref":"Chord.html#cleanup/1"},{"type":"function","title":"Parameters - Chord.cleanup/1","doc":"- `opts` (Keyword.t): Optional parameters for filtering contexts and deltas (see \"Common Options\").","ref":"Chord.html#cleanup/1-parameters"},{"type":"function","title":"Returns - Chord.cleanup/1","doc":"- `:ok` on success.","ref":"Chord.html#cleanup/1-returns"},{"type":"function","title":"Chord.delete_context/1","doc":"Deletes the global context, including all deltas.","ref":"Chord.html#delete_context/1"},{"type":"function","title":"Parameters - Chord.delete_context/1","doc":"- `context_id` - The identifier for the context.","ref":"Chord.html#delete_context/1-parameters"},{"type":"function","title":"Returns - Chord.delete_context/1","doc":"- `:ok` on success.","ref":"Chord.html#delete_context/1-returns"},{"type":"function","title":"Chord.export_context/1","doc":"Exports the active context for a given identifier to external storage using the configured export callback.","ref":"Chord.html#export_context/1"},{"type":"function","title":"Parameters - Chord.export_context/1","doc":"- `context_id` - The identifier for the context.","ref":"Chord.html#export_context/1-parameters"},{"type":"function","title":"Returns - Chord.export_context/1","doc":"- `:ok` if the context is successfully exported.\n- `{:error, :not_found}` if the context does not exist.","ref":"Chord.html#export_context/1-returns"},{"type":"function","title":"Chord.get_context/1","doc":"Retrieves the current global context for a given identifier.","ref":"Chord.html#get_context/1"},{"type":"function","title":"Parameters - Chord.get_context/1","doc":"- `context_id` - The identifier for the context.","ref":"Chord.html#get_context/1-parameters"},{"type":"function","title":"Returns - Chord.get_context/1","doc":"- `{:ok, map()}` on success.\n- `{:error, term()}` on failure.","ref":"Chord.html#get_context/1-returns"},{"type":"function","title":"Chord.restore_context/1","doc":"Restores a context from an external provider to the current backend.","ref":"Chord.html#restore_context/1"},{"type":"function","title":"Parameters - Chord.restore_context/1","doc":"- `context_id` (any): The ID of the context to restore.","ref":"Chord.html#restore_context/1-parameters"},{"type":"function","title":"Returns - Chord.restore_context/1","doc":"- `{:ok, map()}` on success.\n  - `{:error, :not_found}` if the context is missing in external storage.","ref":"Chord.html#restore_context/1-returns"},{"type":"function","title":"Chord.set_context/2","doc":"Sets the global context and records deltas.","ref":"Chord.html#set_context/2"},{"type":"function","title":"Parameters - Chord.set_context/2","doc":"- `context_id` - The identifier for the context.\n  - `new_context` - The new context to be stored.","ref":"Chord.html#set_context/2-parameters"},{"type":"function","title":"Returns - Chord.set_context/2","doc":"- `{:ok, %{context: map(), delta: map()}}` on success.\n- `{:error, term()}` on failure.","ref":"Chord.html#set_context/2-returns"},{"type":"function","title":"Chord.start_cleanup_server/1","doc":"Starts the Cleanup.Server for periodic cleanup.","ref":"Chord.html#start_cleanup_server/1"},{"type":"function","title":"Parameters - Chord.start_cleanup_server/1","doc":"- `opts` - Options for the Cleanup.Server. Examples include:\n    - `:interval` - The interval (in milliseconds) between cleanup executions.\n    - `:backend_opts` - A keyword list of options passed to backend listing functions (see \"Common Options\").","ref":"Chord.html#start_cleanup_server/1-parameters"},{"type":"function","title":"Returns - Chord.start_cleanup_server/1","doc":"- `{:ok, pid}` on success.","ref":"Chord.html#start_cleanup_server/1-returns"},{"type":"function","title":"Chord.stop_cleanup_server/0","doc":"Stops the CleanupServer if running.","ref":"Chord.html#stop_cleanup_server/0"},{"type":"function","title":"Returns - Chord.stop_cleanup_server/0","doc":"- `:ok` on success.","ref":"Chord.html#stop_cleanup_server/0-returns"},{"type":"function","title":"Chord.sync_context/2","doc":"Synchronizes the context for a client based on its current version.","ref":"Chord.html#sync_context/2"},{"type":"function","title":"Parameters - Chord.sync_context/2","doc":"- `context_id` - The identifier for the context.\n  - `client_version` - The last known version for the client.","ref":"Chord.html#sync_context/2-parameters"},{"type":"function","title":"Returns - Chord.sync_context/2","doc":"- `{:full_context, map()}` if a full context is sent.\n- `{:delta, map()}` if deltas are sent.\n- `{:no_change, version}` if no changes are needed.","ref":"Chord.html#sync_context/2-returns"},{"type":"function","title":"Chord.update_cleanup_backend_opts/1","doc":"Updates the backend options for the cleanup server at runtime.","ref":"Chord.html#update_cleanup_backend_opts/1"},{"type":"function","title":"Parameters - Chord.update_cleanup_backend_opts/1","doc":"- `new_opts` (Keyword.t): The new backend options.","ref":"Chord.html#update_cleanup_backend_opts/1-parameters"},{"type":"function","title":"Chord.update_cleanup_interval/1","doc":"Updates the cleanup server's interval at runtime.","ref":"Chord.html#update_cleanup_interval/1"},{"type":"function","title":"Parameters - Chord.update_cleanup_interval/1","doc":"- `new_interval` (integer): The new interval in milliseconds.","ref":"Chord.html#update_cleanup_interval/1-parameters"},{"type":"function","title":"Chord.update_context/2","doc":"Partially updates the global context for a given identifier.","ref":"Chord.html#update_context/2"},{"type":"function","title":"Parameters - Chord.update_context/2","doc":"- `context_id` - The identifier for the context.\n  - `changes` - A map of fields to be updated in the existing context.","ref":"Chord.html#update_context/2-parameters"},{"type":"function","title":"Returns - Chord.update_context/2","doc":"- `{:ok, %{context: map(), delta: map()}}` on success.\n- `{:error, term()}` on failure.","ref":"Chord.html#update_context/2-returns"},{"type":"behaviour","title":"Chord.Backend.Behaviour","doc":"Defines the behavior for backends used by Chord.\n\nThis module specifies the contract that any backend implementation must adhere to.\nIt provides a consistent interface for managing context and delta data, supporting\noperations for context storage, delta tracking, and cleanup.","ref":"Chord.Backend.Behaviour.html"},{"type":"behaviour","title":"Key Concepts - Chord.Backend.Behaviour","doc":"- **Context Operations:** Manage context data associated with a unique `context_id`.\n- **Delta Operations:** Track and retrieve changes (deltas) associated with contexts.\n- **Listing Operations:** Enable filtering and listing of contexts and deltas.\n- **Cleanup:** Support for deleting old or unnecessary data to maintain performance.\n\nDevelopers can implement this behavior to provide custom backends (e.g., Redis, Database).","ref":"Chord.Backend.Behaviour.html#module-key-concepts"},{"type":"behaviour","title":"Example Usage - Chord.Backend.Behaviour","doc":"To implement a custom backend, define a module and implement the callbacks specified here:\n\n    defmodule MyApp.CustomBackend do\n      @behaviour Chord.Backend.Behaviour\n\n      def set_context(context_id, context, version), do: # implementation\n      def get_context(context_id), do: # implementation\n      def delete_context(context_id), do: # implementation\n      # Implement all other callbacks...\n    end\n\nOnce implemented, configure the backend in your application:\n\n    config :chord, :backend, MyApp.CustomBackend","ref":"Chord.Backend.Behaviour.html#module-example-usage"},{"type":"callback","title":"Chord.Backend.Behaviour.delete_context/1","doc":"Deletes the context for a given `context_id`.","ref":"Chord.Backend.Behaviour.html#c:delete_context/1"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.delete_context/1","doc":"- `context_id` (any): The identifier for the context to delete.","ref":"Chord.Backend.Behaviour.html#c:delete_context/1-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.delete_context/1","doc":"- `:ok` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:delete_context/1-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.delete_deltas_by_time/2","doc":"Deletes deltas older than a specified time for a given `context_id`.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_by_time/2"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.delete_deltas_by_time/2","doc":"- `context_id` (any): The identifier for the context.\n  - `older_than_time` (integer): The timestamp threshold.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_by_time/2-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.delete_deltas_by_time/2","doc":"- `:ok` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_by_time/2-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.delete_deltas_exceeding_threshold/2","doc":"Deletes deltas exceeding a specified version threshold for a given `context_id`.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_exceeding_threshold/2"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.delete_deltas_exceeding_threshold/2","doc":"- `context_id` (any): The identifier for the context.\n  - `version_threshold` (integer): The maximum number of versions to retain.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_exceeding_threshold/2-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.delete_deltas_exceeding_threshold/2","doc":"- `:ok` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_exceeding_threshold/2-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.delete_deltas_for_context/1","doc":"Deletes all deltas for a given `context_id`.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_for_context/1"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.delete_deltas_for_context/1","doc":"- `context_id` (any): The identifier for the context.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_for_context/1-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.delete_deltas_for_context/1","doc":"- `:ok` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_for_context/1-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.get_context/1","doc":"Retrieves the context and version for a given `context_id`.","ref":"Chord.Backend.Behaviour.html#c:get_context/1"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.get_context/1","doc":"- `context_id` (any): The identifier for the context.","ref":"Chord.Backend.Behaviour.html#c:get_context/1-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.get_context/1","doc":"- `{:ok, %{context_id: any(), context: map(), version: integer(), inserted_at: integer()}}` on success.\n- `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:get_context/1-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.get_deltas/2","doc":"Retrieves deltas for a given `context_id` greater than a specified `client_version`.","ref":"Chord.Backend.Behaviour.html#c:get_deltas/2"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.get_deltas/2","doc":"- `context_id` (any): The identifier for the context.\n  - `client_version` (integer): The version from which to retrieve deltas.","ref":"Chord.Backend.Behaviour.html#c:get_deltas/2-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.get_deltas/2","doc":"- `{:ok, list(map())}` on success.\n  - `{:error, term()}` if no deltas are found or an error occurs.","ref":"Chord.Backend.Behaviour.html#c:get_deltas/2-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.list_contexts/1","doc":"Lists contexts with optional filters and pagination.","ref":"Chord.Backend.Behaviour.html#c:list_contexts/1"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.list_contexts/1","doc":"- `opts` (Keyword.t): Options for filtering and pagination. Common options include:\n    - `:context_id` - Filter by context identifier.\n    - `:version` - Filter by version.\n    - `:inserted_at` - Filter by insertion timestamp.\n    - `:limit` - Maximum number of results to return.\n    - `:offset` - Number of results to skip.","ref":"Chord.Backend.Behaviour.html#c:list_contexts/1-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.list_contexts/1","doc":"- `{:ok, list(map())}` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:list_contexts/1-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.list_contexts_with_delta_counts/1","doc":"Lists contexts along with their delta counts.","ref":"Chord.Backend.Behaviour.html#c:list_contexts_with_delta_counts/1"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.list_contexts_with_delta_counts/1","doc":"- `opts` (Keyword.t): Options for filtering and pagination.","ref":"Chord.Backend.Behaviour.html#c:list_contexts_with_delta_counts/1-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.list_contexts_with_delta_counts/1","doc":"- `{:ok, list(map())}` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:list_contexts_with_delta_counts/1-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.list_deltas/1","doc":"Lists deltas with optional filters and pagination.","ref":"Chord.Backend.Behaviour.html#c:list_deltas/1"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.list_deltas/1","doc":"- `opts` (Keyword.t): Options for filtering and pagination. Common options include:\n    - `:context_id` - Filter by context identifier.\n    - `:version` - Filter by version.\n    - `:inserted_at` - Filter by insertion timestamp.\n    - `:limit` - Maximum number of results to return.\n    - `:offset` - Number of results to skip.","ref":"Chord.Backend.Behaviour.html#c:list_deltas/1-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.list_deltas/1","doc":"- `{:ok, list(map())}` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:list_deltas/1-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.set_context/3","doc":"Sets the context for a given `context_id`.","ref":"Chord.Backend.Behaviour.html#c:set_context/3"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.set_context/3","doc":"- `context_id` (any): The identifier for the context.\n  - `context` (map): The context data to store.\n  - `version` (integer): The version number associated with the context.","ref":"Chord.Backend.Behaviour.html#c:set_context/3-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.set_context/3","doc":"- `{:ok, %{context_id: any(), context: map(), version: integer(), inserted_at: integer()}}` on success.\n- `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:set_context/3-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.set_delta/3","doc":"Sets a delta for a given `context_id` and `version`.","ref":"Chord.Backend.Behaviour.html#c:set_delta/3"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.set_delta/3","doc":"- `context_id` (any): The identifier for the context.\n  - `delta` (map): The delta data to store.\n  - `version` (integer): The version number associated with the delta.","ref":"Chord.Backend.Behaviour.html#c:set_delta/3-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.set_delta/3","doc":"- `{:ok, %{context_id: any(), delta: map(), version: integer(), inserted_at: integer()}}` on success.\n- `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:set_delta/3-returns"},{"type":"module","title":"Chord.Backend.ETS","doc":"ETS-based backend for Chord, providing context-level state and delta management\nwith versioning support. Both tables use `:ordered_set` to leverage efficient\nkey ordering for listing and cleanup operations.","ref":"Chord.Backend.ETS.html"},{"type":"module","title":"Tables - Chord.Backend.ETS","doc":"- `@context_table`: Stores context state with a composite key `{context_id, inserted_at}`.\n- `@context_history_table`: Stores deltas with a composite key `{context_id, version}`.","ref":"Chord.Backend.ETS.html#module-tables"},{"type":"module","title":"Configuration - Chord.Backend.ETS","doc":"This backend can be used out-of-the-box with the following settings:\n    config :chord, :backend, Chord.Backend.ETS","ref":"Chord.Backend.ETS.html#module-configuration"},{"type":"module","title":"Chord.Backend.Redis","doc":"Redis-based backend for Chord, providing context-level state and delta management\nwith versioning support.","ref":"Chord.Backend.Redis.html"},{"type":"module","title":"Configuration - Chord.Backend.Redis","doc":"To use Redis as a backend, follow these steps:\n\n1. Start a Redis connection process using `Redix` with your desired configuration:\n\n  ```elixir\n  {:ok, _} = Redix.start_link(\"redis://localhost:6379\", name: :my_redis)\n  ```\n\n2. Set the Redis client in the application environment:\n\n  ```elixir\n  Application.put_env(:chord, :redis_client, :my_redis)\n  ```","ref":"Chord.Backend.Redis.html#module-configuration"},{"type":"module","title":"Chord.Cleanup","doc":"This module provides functionality to clean up data that is no longer needed,\nsuch as old contexts or deltas, ensuring efficient use of storage and memory.","ref":"Chord.Cleanup.html"},{"type":"module","title":"Key Features - Chord.Cleanup","doc":"- **Time-Based Cleanup**: Periodically remove stale contexts or deltas based on configurable time-to-live (TTL) values.\n- **Threshold-Based Cleanup**: Retain only the latest N deltas per context, defined by the `:delta_threshold` configuration.\n- **Periodic Cleanup**: A convenient API for batch cleanup of multiple contexts and deltas.","ref":"Chord.Cleanup.html#module-key-features"},{"type":"module","title":"Configuration - Chord.Cleanup","doc":"- `:context_auto_delete` (default: false): Whether to enable automatic context deletion during cleanup.\n- `:context_ttl` (default: nil): Time-to-live for context entries.\n- `:delta_ttl` (default: 24 hours): Time-to-live for deltas.\n- `:delta_threshold` (default: 100): Maximum number of deltas to retain per context.\n- `:backend`: The backend module used for context and delta storage (default: `Chord.Backend.ETS`).","ref":"Chord.Cleanup.html#module-configuration"},{"type":"module","title":"Example Usage - Chord.Cleanup","doc":"iex> Chord.Cleanup.periodic_cleanup(limit: 10)\n    :ok","ref":"Chord.Cleanup.html#module-example-usage"},{"type":"function","title":"Chord.Cleanup.periodic_cleanup/1","doc":"Performs periodic cleanup for contexts and deltas.\n\nThis function fetches all contexts and deltas from the backend, checks their times against\nthe configured TTL values, and removes stale entries.","ref":"Chord.Cleanup.html#periodic_cleanup/1"},{"type":"function","title":"Parameters - Chord.Cleanup.periodic_cleanup/1","doc":"- `opts` (Keyword.t): Optional parameters for fetching contexts and deltas (see \"Common Options\" in the `Chord` module).","ref":"Chord.Cleanup.html#periodic_cleanup/1-parameters"},{"type":"function","title":"Returns - Chord.Cleanup.periodic_cleanup/1","doc":"- `:ok` after performing the cleanup.","ref":"Chord.Cleanup.html#periodic_cleanup/1-returns"},{"type":"function","title":"Example - Chord.Cleanup.periodic_cleanup/1","doc":"iex> Chord.Cleanup.periodic_cleanup(limit: 10)\n    :ok","ref":"Chord.Cleanup.html#periodic_cleanup/1-example"},{"type":"module","title":"Chord.Cleanup.Server","doc":"Periodic cleanup server for stale contexts and deltas.\n\nThis server periodically triggers the cleanup logic defined in `Chord.Cleanup`,\nremoving stale entries from the backend.","ref":"Chord.Cleanup.Server.html"},{"type":"module","title":"State - Chord.Cleanup.Server","doc":"The server's state includes:\n  - `:interval` - The interval (in milliseconds) between cleanup executions.\n  - `:backend_opts` - A keyword list of options passed to backend listing functions.","ref":"Chord.Cleanup.Server.html#module-state"},{"type":"module","title":"Example - Chord.Cleanup.Server","doc":"Chord.Cleanup.Server.start_link(interval: :timer.minutes(30), backend_opts: [limit: 100])","ref":"Chord.Cleanup.Server.html#module-example"},{"type":"function","title":"Chord.Cleanup.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Chord.Cleanup.Server.html#child_spec/1"},{"type":"function","title":"Chord.Cleanup.Server.start_link/1","doc":"Starts the cleanup server with the specified options.","ref":"Chord.Cleanup.Server.html#start_link/1"},{"type":"function","title":"Options - Chord.Cleanup.Server.start_link/1","doc":"- `:interval` (integer): Time interval in milliseconds for periodic cleanup (default: 1 hour).\n  - `:backend_opts` (Keyword.t): Options passed to backend listing functions (see \"Common Options\" in the `Chord` module).","ref":"Chord.Cleanup.Server.html#start_link/1-options"},{"type":"function","title":"Example - Chord.Cleanup.Server.start_link/1","doc":"iex> Chord.Cleanup.Server.start_link(interval: :timer.minutes(30), backend_opts: [limit: 50])\n    {:ok, pid}","ref":"Chord.Cleanup.Server.html#start_link/1-example"},{"type":"function","title":"Chord.Cleanup.Server.update_backend_opts/2","doc":"Updates the backend options for periodic cleanup at runtime.","ref":"Chord.Cleanup.Server.html#update_backend_opts/2"},{"type":"function","title":"Parameters - Chord.Cleanup.Server.update_backend_opts/2","doc":"- `new_opts` (Keyword.t): The new backend options.\n  - `name` (atom): The name of the `GenServer` instance. Defaults to `__MODULE__`.","ref":"Chord.Cleanup.Server.html#update_backend_opts/2-parameters"},{"type":"function","title":"Returns - Chord.Cleanup.Server.update_backend_opts/2","doc":"- `:ok` if the backend options were successfully updated.","ref":"Chord.Cleanup.Server.html#update_backend_opts/2-returns"},{"type":"function","title":"Chord.Cleanup.Server.update_interval/2","doc":"Updates the interval for periodic cleanup at runtime.","ref":"Chord.Cleanup.Server.html#update_interval/2"},{"type":"function","title":"Parameters - Chord.Cleanup.Server.update_interval/2","doc":"- `new_interval` (integer): The new interval in milliseconds.\n  - `name` (atom): The name of the `GenServer` instance. Defaults to `__MODULE__`.","ref":"Chord.Cleanup.Server.html#update_interval/2-parameters"},{"type":"function","title":"Returns - Chord.Cleanup.Server.update_interval/2","doc":"- `:ok` if the interval was successfully updated.","ref":"Chord.Cleanup.Server.html#update_interval/2-returns"},{"type":"module","title":"Chord.Context.Manager","doc":"Orchestrates context synchronization, delta calculation, and backend interactions.\n\nThe `Chord.Context.Manager` module is responsible for the core operations in Chord,\nincluding context management, delta tracking, and backend interactions. It provides\na flexible and efficient system for managing state and deltas, designed to work seamlessly\nwith in-memory or external backends, offering users customizable workflows for their specific needs.","ref":"Chord.Context.Manager.html"},{"type":"module","title":"Features - Chord.Context.Manager","doc":"- **Context Management:** Handles retrieval, updates, and deletions of contexts efficiently.\n- **Delta Tracking:** Automatically calculates and stores deltas between context updates.\n- **Synchronization:** Supports syncing contexts and deltas to clients based on their known versions.\n- **Exporting Contexts:** Allows exporting active contexts to external storage using a configurable callback.\n- **Flexible Backends:** Compatible with built-in ETS or Redis backends, as well as custom backends.\n- **External Context Integration:** Supports fetching contexts from external storage when needed.","ref":"Chord.Context.Manager.html#module-features"},{"type":"module","title":"Use Cases - Chord.Context.Manager","doc":"This module is designed for applications where efficient state and delta management\nis critical, such as:\n\n- Real-time collaborative applications (e.g., document editing, multiplayer games).\n- Communication platforms (e.g., chat systems, call management).\n- Systems requiring versioned state synchronization.\n\nDevelopers can leverage `Chord.Context.Manager` to manage contexts with minimal boilerplate,\nwhile retaining the flexibility to adapt the behavior through configuration and callbacks.","ref":"Chord.Context.Manager.html#module-use-cases"},{"type":"module","title":"Configuration - Chord.Context.Manager","doc":"The behavior of this module is driven by the following configuration options:\n\n    config :chord,\n      backend: Chord.Backend.ETS, # Backend to use for storing contexts and deltas.\n      delta_threshold: 100, # Threshold for determining when to send full contexts.\n      export_callback: &MyApp.ContextExporter.export/1, # Callback for exporting contexts.\n      context_external_provider: &MyApp.ExternalProvider.fetch_context/1 # Function for fetching external contexts.","ref":"Chord.Context.Manager.html#module-configuration"},{"type":"function","title":"Chord.Context.Manager.delete_context/1","doc":"Deletes the context and its associated deltas.","ref":"Chord.Context.Manager.html#delete_context/1"},{"type":"function","title":"Parameters - Chord.Context.Manager.delete_context/1","doc":"- `context_id` (any): The ID of the context to delete.","ref":"Chord.Context.Manager.html#delete_context/1-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.delete_context/1","doc":"- `:ok` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Context.Manager.html#delete_context/1-returns"},{"type":"function","title":"Chord.Context.Manager.export_context/1","doc":"Exports the context for a given identifier to an external storage.","ref":"Chord.Context.Manager.html#export_context/1"},{"type":"function","title":"Parameters - Chord.Context.Manager.export_context/1","doc":"- `context_id` (any): The ID of the context to export.","ref":"Chord.Context.Manager.html#export_context/1-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.export_context/1","doc":"- `:ok` on success.\n  - `{:error, :not_found}` if the context does not exist.","ref":"Chord.Context.Manager.html#export_context/1-returns"},{"type":"function","title":"Chord.Context.Manager.get_context/1","doc":"Retrieves the current context for a given identifier.","ref":"Chord.Context.Manager.html#get_context/1"},{"type":"function","title":"Parameters - Chord.Context.Manager.get_context/1","doc":"- `context_id` (any): The ID of the context to be retrieved.","ref":"Chord.Context.Manager.html#get_context/1-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.get_context/1","doc":"- `{:ok, map()}` if context exists.\n  - `{:error, term()}` if no context is available.","ref":"Chord.Context.Manager.html#get_context/1-returns"},{"type":"function","title":"Chord.Context.Manager.restore_context/1","doc":"Restores a context from an external provider to the current backend.","ref":"Chord.Context.Manager.html#restore_context/1"},{"type":"function","title":"Parameters - Chord.Context.Manager.restore_context/1","doc":"- `context_id` (any): The ID of the context to restore.","ref":"Chord.Context.Manager.html#restore_context/1-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.restore_context/1","doc":"- `{:ok, map()}` on success.\n  - `{:error, :not_found}` if the context is missing in external storage.","ref":"Chord.Context.Manager.html#restore_context/1-returns"},{"type":"function","title":"Chord.Context.Manager.set_context/2","doc":"Updates the global context for a given identifier and calculates deltas.","ref":"Chord.Context.Manager.html#set_context/2"},{"type":"function","title":"Parameters - Chord.Context.Manager.set_context/2","doc":"- `context_id` (any): The ID of the context to update.\n  - `new_context` (map): The full updated context to set.","ref":"Chord.Context.Manager.html#set_context/2-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.set_context/2","doc":"- `{:ok, %{context: map(), delta: map()}}` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Context.Manager.html#set_context/2-returns"},{"type":"function","title":"Chord.Context.Manager.sync_context/2","doc":"Synchronizes the context for a specific identifier.\n\nDetermines whether the client receives the full context, deltas, or no changes\nbased on the client's known version.","ref":"Chord.Context.Manager.html#sync_context/2"},{"type":"function","title":"Parameters - Chord.Context.Manager.sync_context/2","doc":"- `context_id` (any): The ID of the context to synchronize.\n  - `client_version` (integer | nil): The version of the context known to the client.","ref":"Chord.Context.Manager.html#sync_context/2-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.sync_context/2","doc":"- `{:full_context, map()}` if the client should receive the full context.\n  - `{:delta, map()}` if the client should receive deltas.\n  - `{:no_change, integer()}` if the client has the latest version.","ref":"Chord.Context.Manager.html#sync_context/2-returns"},{"type":"function","title":"Chord.Context.Manager.update_context/2","doc":"Partially updates the context for a given identifier and calculates deltas.\n\nOnly the provided fields in `changes` are updated in the existing context.","ref":"Chord.Context.Manager.html#update_context/2"},{"type":"function","title":"Parameters - Chord.Context.Manager.update_context/2","doc":"- `context_id` (any): The ID of the context to update.\n  - `changes` (map): A map of fields to be updated.","ref":"Chord.Context.Manager.html#update_context/2-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.update_context/2","doc":"- `{:ok, %{context: map(), delta: map()}}` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Context.Manager.html#update_context/2-returns"},{"type":"module","title":"Chord.Delta","doc":"Provides utilities for calculating, merging, and formatting context deltas.\n\nThis module contains functions to:\n- Compute the differences (deltas) between two contexts.\n- Merge multiple deltas into a single representation.\n- Format deltas for communication or storage purposes.\n\nThe formatting of deltas is customizable. Developers can define their own formatter module\nimplementing the `Chord.Delta.Formatter.Behaviour` and configure it in their application.\nBy default, the library uses `Chord.Delta.Formatter.Default`.","ref":"Chord.Delta.html"},{"type":"function","title":"Chord.Delta.calculate_delta/2","doc":"Calculates the delta between two contexts.\n\nGiven a `current_context` and a `new_context`, this function determines the differences:\n- Keys present in `new_context` but not in `current_context` are marked as `:added`.\n- Keys present in both but with differing values are marked as `:modified`.\n- Keys present in `current_context` but absent in `new_context` are marked as `:removed`.","ref":"Chord.Delta.html#calculate_delta/2"},{"type":"function","title":"Parameters - Chord.Delta.calculate_delta/2","doc":"- `current_context` (map): The original context.\n  - `new_context` (map): The updated context.","ref":"Chord.Delta.html#calculate_delta/2-parameters"},{"type":"function","title":"Returns - Chord.Delta.calculate_delta/2","doc":"- (map): A map representing the delta, where each key maps to a change descriptor.","ref":"Chord.Delta.html#calculate_delta/2-returns"},{"type":"function","title":"Example - Chord.Delta.calculate_delta/2","doc":"iex> current_context = %{a: 1, b: 2, c: 3}\n    iex> new_context = %{a: 1, b: 5, d: 4}\n    iex> Chord.Delta.calculate_delta(current_context, new_context)\n    %{\n      b: %{action: :modified, old_value: 2, value: 5},\n      c: %{action: :removed},\n      d: %{action: :added, value: 4}\n    }","ref":"Chord.Delta.html#calculate_delta/2-example"},{"type":"function","title":"Chord.Delta.format_delta/2","doc":"Formats a delta map for external communication or storage.\n\nUses the configured delta formatter to transform the delta map into a desired format.\nBy default, it uses `Chord.Delta.Formatter.Default`.","ref":"Chord.Delta.html#format_delta/2"},{"type":"function","title":"Parameters - Chord.Delta.format_delta/2","doc":"- `delta` (map): The delta to format.\n  - `context_id` (any): The context associated with the delta.","ref":"Chord.Delta.html#format_delta/2-parameters"},{"type":"function","title":"Returns - Chord.Delta.format_delta/2","doc":"- (list): The formatted delta, as determined by the formatter module.","ref":"Chord.Delta.html#format_delta/2-returns"},{"type":"function","title":"Example - Chord.Delta.format_delta/2","doc":"iex> delta = %{a: %{action: :added, value: 1}, b: %{action: :removed}}\n    iex> Chord.Delta.format_delta(delta, \"game:1\")\n    [\n      %{key: :a, action: :added, value: 1, context: \"game:1\"},\n      %{key: :b, action: :removed, context: \"game:1\"}\n    ]","ref":"Chord.Delta.html#format_delta/2-example"},{"type":"function","title":"Chord.Delta.merge_deltas/1","doc":"Merges a list of deltas into a single delta.\n\nCombines multiple delta maps to produce a unified view of changes. The merging logic ensures:\n- If any delta marks a key as `:removed`, it takes precedence.\n- If multiple deltas modify the same key, the `old_value` is taken from the first modification,\n  and the `value` from the last.\n- Otherwise, the latest change is retained.","ref":"Chord.Delta.html#merge_deltas/1"},{"type":"function","title":"Parameters - Chord.Delta.merge_deltas/1","doc":"- `delta_list` (list of maps): A list of deltas to merge.","ref":"Chord.Delta.html#merge_deltas/1-parameters"},{"type":"function","title":"Returns - Chord.Delta.merge_deltas/1","doc":"- (map): A single delta representing the merged changes.","ref":"Chord.Delta.html#merge_deltas/1-returns"},{"type":"function","title":"Example - Chord.Delta.merge_deltas/1","doc":"iex> delta1 = %{a: %{action: :added, value: 1}, b: %{action: :modified, old_value: 2, value: 3}}\n    iex> delta2 = %{b: %{action: :removed}, c: %{action: :added, value: 4}}\n    iex> Chord.Delta.merge_deltas([delta1, delta2])\n    %{\n      a: %{action: :added, value: 1},\n      b: %{action: :removed},\n      c: %{action: :added, value: 4}\n    }","ref":"Chord.Delta.html#merge_deltas/1-example"},{"type":"behaviour","title":"Chord.Delta.Formatter.Behaviour","doc":"Defines the behaviour for delta formatters.\n\nThis behaviour allows developers to implement custom delta formatting logic\ntailored to their application's requirements. By adhering to this behaviour,\ndevelopers can ensure that their formatter modules integrate seamlessly with Chord's\ndelta management system.","ref":"Chord.Delta.Formatter.Behaviour.html"},{"type":"behaviour","title":"Example Implementation - Chord.Delta.Formatter.Behaviour","doc":"defmodule MyCustomFormatter do\n      @behaviour Chord.Delta.Formatter.Behaviour\n\n      @impl true\n      def format(delta, context_id) do\n        Enum.map(delta, fn {key, change} ->\n          %{\n            key: key,\n            context: context_id,\n            action: change.action,\n            details: change\n          }\n        end)\n      end\n    end","ref":"Chord.Delta.Formatter.Behaviour.html#module-example-implementation"},{"type":"callback","title":"Chord.Delta.Formatter.Behaviour.format/2","doc":"","ref":"Chord.Delta.Formatter.Behaviour.html#c:format/2"},{"type":"module","title":"Chord.Delta.Formatter.Default","doc":"Default implementation for delta formatting.\n\nThis module provides a straightforward and opinionated way to format deltas into a list of maps.\nEach map describes a change to a specific key in the context, including metadata such as the action\nperformed (e.g., `:added`, `:modified`, `:removed`) and the associated `context_id`.\n\nThe formatting is intended to be extensible, allowing developers to substitute this formatter\nwith a custom implementation by defining their own module and setting it in the application configuration.","ref":"Chord.Delta.Formatter.Default.html"},{"type":"module","title":"Example - Chord.Delta.Formatter.Default","doc":"iex> delta = %{\n    ...>   a: %{action: :added, value: 1},\n    ...>   b: %{action: :modified, old_value: 2, value: 3},\n    ...>   c: %{action: :removed}\n    ...> }\n    iex> context_id = \"game:1\"\n    iex> Chord.Delta.Formatter.Default.format(delta, context_id)\n    [\n      %{key: :a, action: :added, value: 1, context: \"game:1\"},\n      %{key: :b, action: :modified, old_value: 2, value: 3, context: \"game:1\"},\n      %{key: :c, action: :removed, context: \"game:1\"}\n    ]","ref":"Chord.Delta.Formatter.Default.html#module-example"},{"type":"function","title":"Chord.Delta.Formatter.Default.format/2","doc":"Formats the given delta into a standardized list of maps.\n\nThis implementation adds the `context_id` to each change and formats changes as follows:\n- For `:added`, includes the new `value`.\n- For `:modified`, includes both the `old_value` and the new `value`.\n- For `:removed`, includes only the `key` and `context_id`.","ref":"Chord.Delta.Formatter.Default.html#format/2"},{"type":"function","title":"Parameters - Chord.Delta.Formatter.Default.format/2","doc":"- `delta` (map): The delta map representing changes to the context.\n  - `context_id` (any): The context ID to associate with each change.","ref":"Chord.Delta.Formatter.Default.html#format/2-parameters"},{"type":"function","title":"Returns - Chord.Delta.Formatter.Default.format/2","doc":"- (list of maps): A list of formatted changes.","ref":"Chord.Delta.Formatter.Default.html#format/2-returns"},{"type":"function","title":"Example - Chord.Delta.Formatter.Default.format/2","doc":"iex> delta = %{\n    ...>   a: %{action: :added, value: 1},\n    ...>   b: %{action: :modified, old_value: 2, value: 3},\n    ...>   c: %{action: :removed}\n    ...> }\n    iex> Chord.Delta.Formatter.Default.format(delta, \"game:1\")\n    [\n      %{key: :a, action: :added, value: 1, context: \"game:1\"},\n      %{key: :b, action: :modified, old_value: 2, value: 3, context: \"game:1\"},\n      %{key: :c, action: :removed, context: \"game:1\"}\n    ]","ref":"Chord.Delta.Formatter.Default.html#format/2-example"},{"type":"module","title":"Chord.Utils.Context.MapTransform","doc":"Utilities for manipulating nested maps, including deep merging.","ref":"Chord.Utils.Context.MapTransform.html"},{"type":"function","title":"Chord.Utils.Context.MapTransform.deep_update/2","doc":"Updates a deeply nested map using another map of updates.","ref":"Chord.Utils.Context.MapTransform.html#deep_update/2"},{"type":"function","title":"Parameters - Chord.Utils.Context.MapTransform.deep_update/2","doc":"- `original` (map): The original map to be updated.\n- `updates` (map): A map containing the updates to be applied.","ref":"Chord.Utils.Context.MapTransform.html#deep_update/2-parameters"},{"type":"function","title":"Notes - Chord.Utils.Context.MapTransform.deep_update/2","doc":"- Preserves the structure of the original map while applying the updates.\n- Creates missing keys in the original map if they are present in the updates.","ref":"Chord.Utils.Context.MapTransform.html#deep_update/2-notes"},{"type":"function","title":"Examples - Chord.Utils.Context.MapTransform.deep_update/2","doc":"iex> alias Chord.Utils.Context.MapTransform\n    iex> original = %{users: %{user_a: %{name: \"Alice\", age: 30}, user_b: %{name: \"Bob\", age: 25}}}\n    iex> updates = %{users: %{user_b: %{age: 26}}}\n    iex> MapTransform.deep_update(original, updates)\n    %{users: %{user_a: %{name: \"Alice\", age: 30}, user_b: %{name: \"Bob\", age: 26}}}\n\n    iex> original = %{}\n    iex> updates = %{users: %{user_a: %{profile: %{name: \"Alice\"}}}}\n    iex> MapTransform.deep_update(original, updates)\n    %{users: %{user_a: %{profile: %{name: \"Alice\"}}}}","ref":"Chord.Utils.Context.MapTransform.html#deep_update/2-examples"},{"type":"behaviour","title":"Chord.Utils.Redis.Behaviour","doc":"Defines the behavior for Redis client operations.","ref":"Chord.Utils.Redis.Behaviour.html"},{"type":"callback","title":"Chord.Utils.Redis.Behaviour.command/1","doc":"","ref":"Chord.Utils.Redis.Behaviour.html#c:command/1"},{"type":"module","title":"Chord.Utils.Redis.Client","doc":"Wrapper around Redis commands to abstract interaction and enable easier testing.","ref":"Chord.Utils.Redis.Client.html"},{"type":"function","title":"Chord.Utils.Redis.Client.command/1","doc":"Executes a Redis command.\n\nDepending on the configuration, this function will:\n- Interact with the named Redix process (using `Process.whereis/1`).\n- Use a mock module for testing.","ref":"Chord.Utils.Redis.Client.html#command/1"},{"type":"function","title":"Parameters - Chord.Utils.Redis.Client.command/1","doc":"- `command` (list): The Redis command to execute.","ref":"Chord.Utils.Redis.Client.html#command/1-parameters"},{"type":"function","title":"Returns - Chord.Utils.Redis.Client.command/1","doc":"- `{:ok, result}` on success.\n  - `{:error, reason}` on failure.","ref":"Chord.Utils.Redis.Client.html#command/1-returns"},{"type":"module","title":"Chord.Utils.Time","doc":"Default implementation of the `Chord.Utils.Time.Behaviour`.\n\nProvides time utilities for the library, leveraging Erlang's built-in\ntime functions.","ref":"Chord.Utils.Time.html"},{"type":"module","title":"Configuration - Chord.Utils.Time","doc":"By default, this module is used for time-based operations. If you wish to\nuse a custom implementation, define a module adhering to the\n`Chord.Utils.Time.Behaviour` and configure it in your application:\n\n    config :chord, :time, MyCustomTimeProvider","ref":"Chord.Utils.Time.html#module-configuration"},{"type":"module","title":"Example - Chord.Utils.Time","doc":"iex> Chord.Utils.Time.current_time(:second)\n    1672531200","ref":"Chord.Utils.Time.html#module-example"},{"type":"function","title":"Chord.Utils.Time.current_time/1","doc":"Returns the current time in the specified unit.","ref":"Chord.Utils.Time.html#current_time/1"},{"type":"function","title":"Parameters - Chord.Utils.Time.current_time/1","doc":"- `unit` (:second | :millisecond): The desired unit for the time value.","ref":"Chord.Utils.Time.html#current_time/1-parameters"},{"type":"function","title":"Returns - Chord.Utils.Time.current_time/1","doc":"- The current time as an integer, in the specified unit.","ref":"Chord.Utils.Time.html#current_time/1-returns"},{"type":"function","title":"Example - Chord.Utils.Time.current_time/1","doc":"iex> Chord.Utils.Time.current_time(:second)\n    1672531200\n\n    iex> Chord.Utils.Time.current_time(:millisecond)\n    1672531200000","ref":"Chord.Utils.Time.html#current_time/1-example"},{"type":"behaviour","title":"Chord.Utils.Time.Behaviour","doc":"Defines the behavior for time utilities used by the library.\n\nThis behavior allows developers to implement custom time providers,\nmaking it easier to handle time-based operations (e.g., cleanup) and mock\ntime in tests.","ref":"Chord.Utils.Time.Behaviour.html"},{"type":"behaviour","title":"Example - Chord.Utils.Time.Behaviour","doc":"defmodule MyCustomTimeProvider do\n      @behaviour Chord.Utils.Time.Behaviour\n\n      @impl true\n      def current_time(:second) do\n        # Custom logic for returning current time\n        DateTime.utc_now() |> DateTime.to_unix(:second)\n      end\n\n      @impl true\n      def current_time(:millisecond) do\n        # Custom logic for millisecond precision\n        DateTime.utc_now() |> DateTime.to_unix(:millisecond)\n      end\n    end","ref":"Chord.Utils.Time.Behaviour.html#module-example"},{"type":"callback","title":"Chord.Utils.Time.Behaviour.current_time/1","doc":"","ref":"Chord.Utils.Time.Behaviour.html#c:current_time/1"},{"type":"extras","title":"Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"# Chord: Sync, Manage, and Harmonize Contexts 🎵\n\nWelcome to **Chord** — a flexible and powerful Elixir library designed to simplify context management and delta tracking in your distributed or real-time applications.","ref":"readme.html"},{"type":"extras","title":"🎯 Why Chord? - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"When you need a solution for real-time state synchronization, partial updates, and efficient cleanup, Chord strikes the perfect note! Here’s what makes Chord special:\n\n- **Seamless State Sync**: Keep your clients up-to-date with full context or delta-based updates.\n- **Customizable Backend**: Use ETS, Redis, or your own backend implementation.\n- **Flexible Delta Formatting**: Define how your updates are structured.\n- **Periodic Cleanup**: Automatically clear stale contexts or deltas.\n- **Developer-Friendly APIs**: Simple, consistent, and easy-to-use APIs.\n- **Context Export and Restore**: Export contexts to or restore them from external providers.\n- **Partial Updates**: Apply updates to specific fields within a context.\n\n---","ref":"readme.html#why-chord"},{"type":"extras","title":"🚀 Getting Started - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"","ref":"readme.html#getting-started"},{"type":"extras","title":"1️⃣ Install the Library - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"Add Chord to your Mix dependencies:\n\n```elixir\ndef deps do\n  [\n    {:chord, \"~> 0.1.0\"}\n  ]\nend\n```\n\nRun:\n\n```bash\nmix deps.get\n```","ref":"readme.html#1-install-the-library"},{"type":"extras","title":"2️⃣ Configure Chord - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"Add your desired configuration in `config/config.exs`:\n\n```elixir\nconfig :chord,\n  backend: Chord.Backend.ETS,            # Choose your backend (Redis, ETS, etc.)\n  context_auto_delete: false,            # Enable or disable auto-deletion of old contexts\n  context_ttl: :timer.hours(6),          # Time-to-live for contexts\n  delta_ttl: :timer.hours(24),           # Time-to-live for deltas\n  delta_threshold: 100,                  # Number of deltas to retain\n  delta_formatter: Chord.Delta.Formatter.Default, # Format for deltas\n  time_provider: Chord.Utils.Time,       # Time provider for consistent timestamps\n  export_callback: nil,                  # Optional: Define a callback for exporting contexts\n  context_external_provider: nil         # Optional: Define a function for fetching external contexts\n```\n\n---","ref":"readme.html#2-configure-chord"},{"type":"extras","title":"🎹 How to Use Chord - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"","ref":"readme.html#how-to-use-chord"},{"type":"extras","title":"Setting a Context - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"```elixir\n{:ok, result} = Chord.set_context(\"user:123\", %{status: \"online\"})\nIO.inspect(result, label: \"Context Set\")\n```","ref":"readme.html#setting-a-context"},{"type":"extras","title":"Updating a Context - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"```elixir\n{:ok, result} = Chord.update_context(\"user:123\", %{status: \"away\"})\nIO.inspect(result, label: \"Context Updated\")\n```","ref":"readme.html#updating-a-context"},{"type":"extras","title":"Synchronizing State - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"```elixir\ncase Chord.sync_context(\"user:123\", nil) do\n  {:full_context, context} -> IO.inspect(context, label: \"Full Context\")\n  {:delta, delta} -> IO.inspect(delta, label: \"Delta Update\")\n  {:no_change, version} -> IO.puts(\"No changes for version #{version}\")\nend\n```","ref":"readme.html#synchronizing-state"},{"type":"extras","title":"Exporting a Context - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"```elixir\n:ok = Chord.export_context(\"user:123\")\n```","ref":"readme.html#exporting-a-context"},{"type":"extras","title":"Restoring a Context - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"```elixir\n{:ok, restored_context} = Chord.restore_context(\"user:123\")\nIO.inspect(restored_context, label: \"Restored Context\")\n```","ref":"readme.html#restoring-a-context"},{"type":"extras","title":"Cleanup Operations - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"Run periodic cleanup to remove stale data:\n\n```elixir\nChord.cleanup(limit: 50)\n```","ref":"readme.html#cleanup-operations"},{"type":"extras","title":"Managing the Cleanup Server - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"Start and manage the Cleanup Server for automated periodic cleanup:\n\n```elixir\n{:ok, _pid} = Chord.start_cleanup_server(interval: :timer.minutes(30))\nChord.update_cleanup_interval(:timer.minutes(60))\nChord.update_cleanup_backend_opts(limit: 100)\nChord.stop_cleanup_server()\n```\n\n---","ref":"readme.html#managing-the-cleanup-server"},{"type":"extras","title":"🛠️ Customization - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"","ref":"readme.html#customization"},{"type":"extras","title":"Backends - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"Chord supports multiple backends out-of-the-box:\n\n- **ETS** (In-Memory)\n- **Redis** (Distributed)\n\nYou can implement your own backend by adhering to the `Chord.Backend.Behaviour`.","ref":"readme.html#backends"},{"type":"extras","title":"Delta Formatters - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"Customize how deltas are structured by implementing the `Chord.Delta.Formatter` behaviour.","ref":"readme.html#delta-formatters"},{"type":"extras","title":"⚡ Features at a Glance - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"| Feature                  | Description                                      |\n|--------------------------|--------------------------------------------------|\n| **Real-Time Sync**       | Delta-based and full-context synchronization.    |\n| **Customizable Backends**| Redis, ETS, or your own custom backend.          |\n| **Periodic Cleanup**     | Automatically remove stale data.                 |\n| **Partial Updates**      | Update only specific fields in a context.        |\n| **Delta Tracking**       | Efficiently track and retrieve state changes.    |\n| **Context Export**       | Export context to external storage.              |\n| **Context Restore**      | Restore context from external providers.         |\n\n---","ref":"readme.html#features-at-a-glance"},{"type":"extras","title":"🕰 Benchmark Results - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"Chord has been benchmarked using Redis and ETS backends under both stateless and stateful architectures. Below are the results for various scenarios:","ref":"readme.html#benchmark-results"},{"type":"extras","title":"Redis Benchmark Results - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"#### With Input Data\n\n| Scenario                                             | IPS    | Avg. Time | Deviation | Median Time | 99th Percentile |\n|------------------------------------------------------|--------|-----------|-----------|-------------|-----------------|\n| Stateless - Single Context (50 participants)         | 62.79  | 15.93 ms  | ±18.03%   | 15.88 ms    | 22.99 ms        |\n| Stateful - Single Context (50 participants)          | 7.69   | 130.07 ms | ±51.34%   | 102.08 ms   | 270.40 ms       |\n| Stateless - Multiple Contexts (100 contexts)         | 2.08   | 481.38 ms | ±3.48%    | 474.73 ms   | 511.20 ms       |\n| Stateful - Multiple Contexts (100 contexts)          | 1.85   | 541.19 ms | ±3.51%    | 541.24 ms   | 566.32 ms       |\n\n**Comparison:**\n- Stateless - Single Context: 62.79x faster than Stateful - Multiple Contexts.","ref":"readme.html#redis-benchmark-results"},{"type":"extras","title":"ETS Benchmark Results - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"#### With Input Data\n\n| Scenario                                             | IPS    | Avg. Time | Deviation | Median Time | 99th Percentile |\n|------------------------------------------------------|--------|-----------|-----------|-------------|-----------------|\n| Stateless - Single Context (50 participants)         | 151.19 | 6.61 ms   | ±20.47%   | 6.46 ms     | 14.43 ms        |\n| Stateful - Single Context (50 participants)          | 29.56  | 33.82 ms  | ±13.06%   | 36.05 ms    | 45.56 ms        |\n| Stateless - Multiple Contexts (100 contexts)         | 3.51   | 284.85 ms | ±14.28%   | 279.17 ms   | 405.34 ms       |\n| Stateful - Multiple Contexts (100 contexts)          | 3.73   | 268.05 ms | ±23.67%   | 237.10 ms   | 374.59 ms       |\n\n**Comparison:**\n- Stateless - Single Context: 43.07x faster than Stateful - Multiple Contexts.","ref":"readme.html#ets-benchmark-results"},{"type":"extras","title":"Device Information - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"| Property                   | Value                    |\n|----------------------------|--------------------------|\n| **Operating System**       | macOS                    | \n| **CPU Information**        | Apple M4 Pro             |\n| **Number of Cores**        | 12                       |\n| **Available Memory**       | 24 GB                    |\n| **Elixir Version**         | 1.17.3                   |\n| **Erlang Version**         | 27.1.2                   |\n| **JIT Enabled**            | True                     |\n\n**Benchmark Suite Configuration:**\n- **Warmup:** 2 seconds\n- **Execution Time:** 5 seconds\n- **Parallel:** 1\n- **Inputs:** Data\n\n---","ref":"readme.html#device-information"},{"type":"extras","title":"🧰 Contributing - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"Contributions from the community are welcome to make Chord even better! Whether it's fixing bugs, improving documentation, or adding new features, your help is greatly appreciated.","ref":"readme.html#contributing"},{"type":"extras","title":"How to Contribute - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"1. Fork the repository.\n2. Create a new branch for your changes.\n3. Make your changes and test them thoroughly.\n4. Submit a pull request with a clear description of your changes.\n\nFeel free to open issues for discussion or if you need help. Together, we can build something amazing!\n\n---","ref":"readme.html#how-to-contribute"},{"type":"extras","title":"🛡️ Testing - Chord: Sync, Manage, and Harmonize Contexts 🎵","doc":"Chord comes with a robust suite of tests to ensure reliability. Run tests with:\n\n```bash\nmix test\n```\n\n---\n\n🎵 *\"Let Chord orchestrate your state management with precision and elegance.\"*","ref":"readme.html#testing"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/).\n\n## [0.1.1] - 2025-01-05","ref":"changelog.html"},{"type":"extras","title":"Fixed - Changelog","doc":"- Corrected Redis configuration example in documentation.\n- Fixed an issue in `Chord.Delta.calculate_delta/2` to correctly handle nested maps and avoid returning empty maps for unchanged nested structures.\n\n## [0.1.0] - 2024-12-31","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Initial release of the Chord library.\n- Real-time state synchronization with full and delta-based updates.\n- Support for ETS and Redis backends.\n- Developer-friendly APIs for setting, updating, and syncing contexts.\n- Periodic cleanup of stale contexts and deltas.\n- Support for context export and restore.\n- Benchmark results showcasing performance for ETS and Redis under stateless and stateful architectures.","ref":"changelog.html#added"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}